Unit Frame_Config;

Interface

Uses
    Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
    Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls;

Type
    TBitFrame = Class( TFrame )
        Panel1 : TPanel;
        BitNameLabel : TLabel;
        IODirectionGroup : TRadioGroup;
        LoPulseEdit : TLabeledEdit;
        HiPulseEdit : TLabeledEdit;
        DutyCycleEdit : TLabeledEdit;
        EventDetection : TRadioGroup;
        EventCounterEdit : TLabeledEdit;
        FrequencyEdit : TLabeledEdit;
        Private
            fIOGroup : Byte;
            fBitIndex : Byte;
            fRegisterBank : Word;
            Procedure setIOGroup( Const Value : Byte );
            Procedure setBitIndex( Const Value : Byte );
            Procedure setRegisterBank( Const Value : Word );
            Procedure UpdateNameLabel;
            Function CountsToFrequency( Count : LongWord ) : double;
            Procedure DisplayFrequency;
            Procedure DisplayPulseWidth;
            Procedure DisplayPWM;
            Procedure GetRegistersAsStrings( BitIndex : Integer; regs : TStringList );
            { Private declarations }
        Public
            { Public declarations }
            Procedure UpdateReadings;
        Published
            Property IOGroup : Byte Read fIOGroup Write setIOGroup;
            Property BitIndex : Byte Read fBitIndex Write setBitIndex;
            Property RegOffset : Word Read fRegisterBank Write setRegisterBank;
    End;

Function OffsetForBitIndex( BitIndex : Integer ) : Integer;

Const
    margin                    = 2;
    maxText                   = 'bit#44L/H = WWWWWs/WWWWWs';
    REG_CONFIGURATION         = $00;
    REG_EVENTCOUNTER          = $08;
    REG_EVENTCOUNTERTHRESHOLD = $0C;
    REG_FREQUENCY             = $1C;
    REG_PULSEDURATIONLO       = $20;
    REG_PULSEDURATIONHI       = $24;
    REG_PWMLO                 = $30;
    REG_PWMHI                 = $34;

    bmEnableEventH2L          = $00000001;                                 // count falling edges
    bmEnableEventL2H          = $00000002;                                 // count rising edges
    bmEnableEventLoPulse      = $00000004;                                 // count low-going pulses
    bmEnableEventHiPulse      = $00000008;                                 // count high-going pulses
    bmMeasurePulseWidths      = $00000010;                                 // measure high- and low-side pulse widths
    bmMeasureDutyCycle        = $00000020;                                 // measure PWM duty cycle
    bmMeasureFrequency        = $00000040;                                 // measure frequency (only 4 bits support this)
    bmDebounce                = $00000080;                                 // apply a 1ms debounce digital filter before using the input for anything else

    bmOutputPulseTrain        = $00010000;                                 // generate a pulse train on this bit
    bmOutputPWM               = $00020000;                                 // generate a PWM on this bit (only 4 bits support this: why?)
    bmOutputPulsesDone        = $00040000;                                 // indicate "train of pulses has finished being output"
    bmEnableIRQ               = $00080000;                                 // enable Events to generate IRQs for this bit


Implementation

{$R *.dfm}

Uses
    MainUnit, AIOWDM;
{ TBitFrame }

Function OffsetForBitIndex( BitIndex : Integer ) : Integer;
Begin
    Result := $100 + ( BitIndex * $100 );
End;


Function BitIndexToPort( BitIndex : Integer ) : String;
Begin
    Result := chr( BitIndex Div 8 + ord( 'A' ) );
End;


Function TBitFrame.CountsToFrequency( Count : LongWord ) : double;
Begin
    If Count = 0 Then
        Result := 0.0
    Else
        Result := 1 / ( Count * 0.000000008 );
End;


Procedure TBitFrame.DisplayPulseWidth;
Var
    BitIndex : Integer;
    PulseLoDurationus, PulseHiDurationus : real;
    PulseLoDuration : Word;
    PulseHiDuration : Word;
    itemsPerRow : Integer;
Begin
    PulseLoDuration := RelInPort( MainForm.CardNum, OffsetForBitIndex( fBitIndex ) + REG_PULSEDURATIONLO );
    PulseHiDuration := RelInPort( MainForm.CardNum, OffsetForBitIndex( fBitIndex ) + REG_PULSEDURATIONHI );

    PulseLoDurationus := PulseLoDuration * 8.0;
    PulseHiDurationus := PulseHiDuration * 8.0;

    If PulseLoDurationus > 1000 Then
        LoPulseEdit.Text := Format( '%0.0fms', [PulseLoDurationus / 1000] )
    Else
        LoPulseEdit.Text := Format( '%0.0fs', [PulseLoDurationus] );
    If PulseHiDurationus > 1000 Then
        HiPulseEdit.Text := Format( '%0.0fms', [PulseHiDurationus / 1000] )
    Else
        HiPulseEdit.Text := Format( '%0.0fs', [PulseHiDurationus] );
End;


Procedure TBitFrame.DisplayFrequency;
Var
    Count : LongWord;
    frequency : double;
    itemsPerRow : Integer;
    msg : String;
Begin
    Count := RelInPortL( MainForm.CardNum, OffsetForBitIndex( fBitIndex ) + REG_FREQUENCY );
    frequency := CountsToFrequency( Count );

    If frequency = 0.0 Then
        msg := '- -'
    Else
        If ( frequency >= 1000000 ) Then
            msg := floattostr( Round( frequency / 1000 ) / 1000 ) + ' MHz' // Display value in MHz
        Else
            If ( frequency >= 1000 ) Then
                msg := floattostr( Round( frequency ) / 1000 ) + ' KHz'    // Display value in KHz
            Else
                msg := floattostr( Round( frequency ) ) + ' Hz';           // Display value in Hz

    FrequencyEdit.Text := Format( '%s', [msg] );
End;


Procedure TBitFrame.DisplayPWM;
Var
    PulseLoDuration : Word;
    PulseHiDuration : Word;
    itemsPerRow : Integer;
Begin
    PulseLoDuration := RelInPort( MainForm.CardNum, OffsetForBitIndex( fBitIndex ) + REG_PWMLO );
    PulseHiDuration := RelInPort( MainForm.CardNum, OffsetForBitIndex( fBitIndex ) + REG_PWMHI );

    If PulseHiDuration <> 0 Then
        DutyCycleEdit.Text := Format( '%f%%', [PulseHiDuration / ( PulseHiDuration + PulseLoDuration ) * 100] )
    Else
        DutyCycleEdit.Text := Format( '- -', [fBitIndex] );
End;


Procedure TBitFrame.GetRegistersAsStrings( BitIndex : Integer; regs : TStringList );
//Var
//    reg : Integer;
Begin
//    For reg := 0 To 9 Do
//    Begin
//        ReadValue := RelInPortL( MainForm.CardNum, OffsetForBitIndex( fBitIndex ) + reg * 4 );
//        If regs.Count < reg + 1 Then
//            regs.Add( IntToHex( ReadValue, 8 ) )
//        Else
//            regs[reg] := IntToHex( ReadValue, 8 );
//    End;
End;


Procedure TBitFrame.UpdateReadings;
Begin
    DisplayPWM;
    DisplayFrequency;
    DisplayPulseWidth;
End;


Procedure TBitFrame.UpdateNameLabel;
Begin
    BitNameLabel.Caption := 'Bit ' + IntToStr( fBitIndex ) + ' (Port ' + BitIndexToPort( fBitIndex ) + IntToStr( fBitIndex Mod 8 ) + ')';
End;


Procedure TBitFrame.setBitIndex( Const Value : Byte );
Begin
    fBitIndex := Value;
    fIOGroup := fBitIndex Div 8;
    UpdateNameLabel;
End;


Procedure TBitFrame.setIOGroup( Const Value : Byte );
Begin
    fIOGroup := Value;
    UpdateNameLabel;
End;


Procedure TBitFrame.setRegisterBank( Const Value : Word );
Begin
    fRegisterBank := Value;
    UpdateNameLabel;
End;

End.
