unit Frame_inputBitConfig;

interface

uses 
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TInputBitConfigFrame = class(TFrame)
    LabelBitNumber: TLabel;
    checkBoxDebounce: TCheckBox;
    cbMeasurement: TComboBox;
    cbEventCounter: TComboBox;
    procedure bitConfigurationClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

function OffsetForBitIndex(bitIndex: Integer): integer;

const
  margin = 2;
  maxText = 'bit#44L/H = WWWWWs/WWWWWs';
  REG_CONFIGURATION = $0;
  REG_EVENTCOUNTER  = $4;
  REG_PULSEDURATION = $8;
  REG_PWM           = $C;
  REG_FREQUENCY     = $14;

  bmEnableEventH2L     = $00000001; // count falling edges
  bmEnableEventL2H     = $00000002; // count rising edges
  bmEnableEventLoPulse = $00000004; // count low-going pulses
  bmEnableEventHiPulse = $00000008; // count high-going pulses
  bmMeasurePulseWidths = $00000010; // measure high- and low-side pulse widths
  bmMeasureDutyCycle   = $00000020; // measure PWM duty cycle
  bmMeasureFrequency   = $00000040; // measure frequency (only 4 bits support this)
  bmDebounce           = $00000080; // apply a 1ms debounce digital filter before using the input for anything else

  bmOutputPulseTrain = $00010000; // generate a pulse train on this bit
  bmOutputPWM        = $00020000; // generate a PWM on this bit (only 4 bits support this: why?)
  bmOutputPulsesDone = $00040000; // indicate "train of pulses has finished being output"
  bmEnableIRQ        = $00080000; // enable Events to generate IRQs for this bit

implementation

{$R *.DFM}

uses AIOWDM, MainUnit;

function OffsetForBitIndex(bitIndex: Integer): integer;
begin
  Result := $100 + (bitIndex * $100);
end;

procedure TInputBitConfigFrame.bitConfigurationClick(Sender: TObject);
  var
    ReadValue, WriteValue: Longword;
begin
  ReadValue := RelInPortL(Mainform.CardCombo.ItemIndex, OffsetForBitIndex(0) + REG_CONFIGURATION) and $FFFF0000; // elminate previous "input" config bits
  WriteValue := ReadValue;

  if checkBoxDebounce.Checked then WriteValue := ReadValue or bmDebounce;

  case cbMeasurement.ItemIndex of
  1: WriteValue := WriteValue or bmMeasurePulseWidths;
  2: WriteValue := WriteValue or bmMeasureDutyCycle;
  3: WriteValue := WriteValue or bmMeasureFrequency;
  end;

  case cbEventCounter.ItemIndex of
  1: WriteValue := WriteValue or bmEnableEventL2H;
  2: WriteValue := WriteValue or bmEnableEventH2L;
  3: WriteValue := WriteValue or bmEnableEventHiPulse;
  4: WriteValue := WriteValue or bmEnableEventLoPulse;
  end;

  RelOutPortL(Mainform.CardCombo.ItemIndex, OffsetForBitIndex(0) + REG_CONFIGURATION, WriteValue);
end;


end.
