Unit MainUnit;

Interface

Uses
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    StdCtrls, ExtCtrls, ComCtrls;

Type
    TMainForm = Class( TForm )
        Panel1 : TPanel;
        Timer1 : TTimer;
        CardCombo : TComboBox;
        OffPanel : TPanel;
        Label1 : TLabel;
        OffEdit : TEdit;
        DataPanel : TPanel;
        Label2 : TLabel;
        DataEdit : TEdit;
        MemGroup : TGroupBox;
        MWrite8Button : TButton;
        MWrite16Button : TButton;
        MWrite32Button : TButton;
        MRead8Button : TButton;
        MRead16Button : TButton;
        MRead32Button : TButton;
        LogMemo : TMemo;
        FlowPanel1 : TFlowPanel;
    ButtonShowPulseMeasurements: TButton;
        Procedure FormCreate( Sender : TObject );
        Procedure FormResize( Sender : TObject );
        Procedure WriteButtonClick( Sender : TObject );
        Procedure ReadButtonClick( Sender : TObject );
        Procedure ButtonShowPulseMeasurementsClick( Sender : TObject );
        Procedure Timer1Timer( Sender : TObject );

        Protected
            Procedure LogTransaction( bIO, bRead : Boolean; Bits, Offset, Data : LongWord; Status : Word );
            { Private declarations }
        Public
            CardNum : LongInt;
            { Public declarations }
    End;

Var
    MainForm : TMainForm;

Implementation

{$R *.DFM}

Uses
    AIOWDM, Frame_Config;


Procedure LoadDefaultFont( IntoFont : TFont );
Var
    FontSize : Integer;
    NCM : TNonClientMetrics;

Begin
    NCM.cbSize := SizeOf( NCM );
    If SystemParametersInfo( SPI_GETNONCLIENTMETRICS, SizeOf( NCM ), @NCM, 0 ) Then
    Begin
        FontSize := Round( - 3 / 4 * NCM.lfMessageFont.lfHeight );
        IntoFont.Name := NCM.lfMessageFont.lfFaceName;
    End
    Else
    Begin
        FontSize := Abs( IntoFont.Size );
        IntoFont.Name := 'Tahoma';
    End;
    IntoFont.PixelsPerInch := 72;
    IntoFont.Size := FontSize;
End;


Function AnsiMessageBox( Text, Caption : AnsiString; uType : UINT ) : Integer;
Begin
    Result := MessageBoxA( Application.Handle, PAnsiChar( Text ), PAnsiChar( Caption ), uType );
    Text := '';
    Caption := '';
End;


Procedure TMainForm.FormCreate( Sender : TObject );
Var
    I : Integer;
    Nom : AnsiString;
    NomSize, DeviceID, Status : LongWord;

Begin
    LoadDefaultFont( Font );
    LogMemo.Font.Name := 'Lucida Console';

    For I := 0 To GetNumCards - 1 Do
    Begin
        NomSize := 256;
        SetString( Nom, Nil, NomSize );
        If QueryCardInfo( I, @DeviceID, Nil, @NomSize, @Nom[1] ) <> 0 Then
        Begin
            If NomSize <> 0 Then
                Dec( NomSize );
            SetLength( Nom, NomSize );
            CardCombo.Items.Add( '[' + IntToStr( I ) + '] ' + Nom + ' [' + IntToHex( DeviceID, 4 ) + ']' );
        End
        Else
        Begin
            Status := GetLastError;
            Nom := '';
            CardCombo.Items.Add( '[' + IntToStr( I ) + '] Error ' + IntToStr( Status ) + ' Querying Card' );
        End;
    End;
    If CardCombo.Items.Count <> 0 Then
        CardCombo.ItemIndex := 0;
End;


Procedure TMainForm.FormResize( Sender : TObject );
Var
    W2, H, T : Integer;

Begin
    W2 := ClientWidth Div 2;
    With OffPanel.BoundsRect Do
    Begin
        T := Top;
        H := Bottom - T;
    End;
    OffPanel.SetBounds( 0, T, W2, H );
    DataPanel.SetBounds( W2, T, W2, H );
End;


Procedure TMainForm.WriteButtonClick( Sender : TObject );
Var
    CardNum, Bits : LongInt;
    Offset : Word;
    Data : LongWord;
    bIO : Boolean;
    Status : Word;

Begin
    CardNum := CardCombo.ItemIndex;
    Offset := StrToInt( '$' + OffEdit.Text );
    Data := StrToInt( '$' + DataEdit.Text );
    With Sender As TButton Do
    Begin
        Bits := Tag;
        bIO := False;
    End;

    Status := RelChangeMapping( CardNum, bIO );

    LogTransaction( bIO, False, Bits, Offset, Data, Status );



    Case Bits Of
        8 :
            RelOutPortB( CardNum, Offset, Data );
        16 :
            RelOutPort( CardNum, Offset, Data );
        32 :
            RelOutPortL( CardNum, Offset, Data );
    End;
End;


Procedure TMainForm.ReadButtonClick( Sender : TObject );
Var
    CardNum, Bits : LongInt;
    Offset : Word;
    Data : LongWord;
    bIO : Boolean;
    Status : Word;

Begin
    CardNum := CardCombo.ItemIndex;
    Offset := StrToInt( '$' + OffEdit.Text );
    With Sender As TButton Do
    Begin
        Bits := Tag;
        bIO := False;
    End;

    Status := RelChangeMapping( CardNum, bIO );
    Case Bits Of
        8 :
            Data := RelInPortB( CardNum, Offset );
        16 :
            Data := RelInPort( CardNum, Offset );
        32 :
            Data := RelInPortL( CardNum, Offset );
        Else
            Exit;
    End;

    LogTransaction( bIO, True, Bits, Offset, Data, Status );

    DataEdit.Text := IntToHex( Data, Bits Div 4 );
End;


Procedure TMainForm.LogTransaction( bIO, bRead : Boolean; Bits, Offset, Data : LongWord; Status : Word );
Var
    Buf : AnsiString;

Begin
    // This IF is needed because of an oddity of the chip: SHL 32 does nothing, instead of zeroing out.
    If Bits < 32 Then
        Data := Data And Not ( - 1 Shl Bits );

    If bIO Then
        Buf := 'I/O +'
    Else
        Buf := 'Mem +';
    Buf := Buf + IntToHex( Offset, 1 );
    If bRead Then
        Buf := Buf + ' > '
    Else
        Buf := Buf + ' < ';
    Buf := Buf + IntToHex( Data, Bits Div 4 ) + ' Status: ' + IntToHex( Status, 4 );


    LogMemo.Lines.Add( Buf );
End;


Procedure TMainForm.ButtonShowPulseMeasurementsClick( Sender : TObject );
Var
    Frame : TBitFrame;
    n : Integer;
Begin

    If Timer1.Enabled = True Then
    Begin
        Timer1.Enabled := False;
        ButtonShowPulseMeasurements.Caption := '&Start Reading';
    End
    Else
    Begin
        For n := 0 To 23 Do
        Begin
            Frame := TBitFrame.Create( self );
            Frame.Parent := FlowPanel1;
            Frame.Visible := True;
            Frame.Name := 'BitFrame' + IntToStr( n );
            Frame.BitIndex := n;
            if (n div 8) = 1 then
              Frame.Color := clBackground;
        End;

        Timer1.Enabled := True;
        ButtonShowPulseMeasurements.Caption := '&Stop Reading';
    End;
End;


Procedure TMainForm.Timer1Timer( Sender : TObject );
Var
    BitIndex : Integer;
    Frame: TBitFrame;
Begin
    CardNum := CardCombo.ItemIndex;
    For BitIndex := 0 To 23 Do
    Begin
        Frame := FlowPanel1.Controls[BitIndex] as TBitFrame;
        Frame.updateReadings;
    End;


End;
{
  can't enable both L2H and H2L for any given bit
  can enable Pulse H/L and L2H or H2L
  must write 32-bits to $104, can't write 8-bits to $106:
}


End.
