﻿/**
  ******************************************************************************
  * @file    DetectPulseConfigWinForm.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file handles the Pulse detect configuration dialog box
  ******************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace mPCIe_DIO_24s
{    
    public partial class DetectPulseConfigWinForm : Form
    {
        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the DetectPulseConfigWinForm components
        /// </summary>
        public DetectPulseConfigWinForm()
        {
            InitializeComponent();
        }

        /*************************************************************************************************************
         * 
         * Private Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Form Load event, Load the previously configured parameters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void DetectPulseConfigWinForm_Load(object sender, EventArgs e)
        {
            // Load the previous configured values
            Tuple<ePortName, ePinNum> pin;
            pin = ucPinConfig.GetLastUpdatedPortPinName();

            // If already configured, load the previously configured values
            if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseDetectConfig.TriggerType == eConfigParam.eRisingEdge)
                rdRisingEdge.Checked = true;
            else if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseDetectConfig.TriggerType == eConfigParam.eFallingEdge)
                rbFallingEdge.Checked = true;
            else if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseDetectConfig.TriggerType == eConfigParam.eHighPulse)
                rbHighPulse.Checked = true;
            else if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseDetectConfig.TriggerType == eConfigParam.eLowPulse)
                rbLowPulse.Checked = true;

            // Check if IRQ is enabled
            if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].PulseDetectConfig.EnableIrq == true)
                cbEnableIRQ.Checked = true;         
        }

        /// <summary>
        /// OK Button click event handler
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btOK_Click(object sender, EventArgs e)
        {
            eConfigParam TriggerType = eConfigParam.eNone;
            ucPinConfig pinconfig = new ucPinConfig();
            bool enableIrq = false;            

            // Get the trigger type
            if (rdRisingEdge.Checked == true)
                TriggerType = eConfigParam.eRisingEdge;            
            else if (rbFallingEdge.Checked == true)
                TriggerType = eConfigParam.eFallingEdge;
            else if (rbHighPulse.Checked == true)
                TriggerType = eConfigParam.eHighPulse;
            else if (rbLowPulse.Checked == true)
                TriggerType = eConfigParam.eLowPulse;

            // Check if IRQ is enabled
            if (cbEnableIRQ.Checked == true)
                enableIrq = true;

            // Update the configuration
            pinconfig.PulseDetectConfigUpdate(TriggerType, enableIrq);
            
            // Close the popup window
            this.Close();            
        }

        /// <summary>
        /// Pulse Detect Configuration window closed event Handler. Dispose the unmanaged resources.
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void DetectPulseConfigWinForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/