﻿/**
  ******************************************************************************
  * @file    PWMGenConfigWinform.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file handles the PWM configuration dialog box
  ******************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace mPCIe_DIO_24s
{
    public partial class PWMGenConfigWinform : Form
    {
        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the PWMGenConfigWinform components
        /// </summary>
        public PWMGenConfigWinform()
        {
            InitializeComponent();
        }

        /*************************************************************************************************************
         * 
         * Private Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// PWM Form Load Event handler. Initializes the configuration with last configured value
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void PWMGenConfigWinform_Load(object sender, EventArgs e)
        {
            // Load the previous configured values
            Tuple<ePortName, ePinNum> pin;
            pin = ucPinConfig.GetLastUpdatedPortPinName();

            // Check if pin is already configured for PWM
            if (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].DutyCycleConfig.TimePeriodMilliSec != 0)
            {
                tbTotalTimePer.Text = (Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].DutyCycleConfig.TimePeriodMilliSec * 1000).ToString();  // Update the total time period textbox
                trkbDutyCycle.Value = Main_Form.ConfigurationParam[(int)pin.Item1, (int)pin.Item2].DutyCycleConfig.DutyCycle;   // Update the Duty cycle track bar
                lbDutyCycleVal.Text = trkbDutyCycle.Value.ToString() + "%";
            }                     
        }

        /// <summary>
        /// Update the track bar information on label lbDutyCycleVal
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void trkbDutyCycle_Scroll(object sender, EventArgs e)
        {
            int i = trkbDutyCycle.Value;
            lbDutyCycleVal.Text = i.ToString() + "%";
        }

        /// <summary>
        /// Allow only numeric values in the text box
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbTotalTimePer_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar);
        }

        /// <summary>
        /// Ok button click event handler. Validate and save the configuration parameters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btOk_Click(object sender, EventArgs e)
        {
            ucPinConfig uc = new ucPinConfig();
            UInt32 totalTimePeriodMicroSec = 0;
            float totalTimePeriodMilliSec = 0;
            int dutyCyclePercentage = 0;

            // Make sure that text box is not empty
            if (tbTotalTimePer.Text == "")
            {
                MessageBox.Show("Please enter total period of the signal");
            }
            else
            {
                try
                {
                    totalTimePeriodMicroSec = UInt32.Parse(tbTotalTimePer.Text);   // Get total time period value
                }
                catch
                {
                    MessageBox.Show("Entered Total time Period data is Invalid or out of range");
                    return;
                }

                // Get duty cycle value
                dutyCyclePercentage = trkbDutyCycle.Value;
                
                // Check if entered value is non zero value
                if (totalTimePeriodMicroSec == 0)
                {
                    MessageBox.Show("Total period should be non-zero value");
                }
                else
                {
                    // Convert total time period uSec value to mSec
                    totalTimePeriodMilliSec = (float)totalTimePeriodMicroSec / 1000;                    

                    // Update the PWM output configuration
                    ucPinConfig.DutyCycleConfigUpdate(totalTimePeriodMilliSec, dutyCyclePercentage);

                    // Close the configuration pop-up window
                    this.Close();
                }                
            }
        }

        /// <summary>
        /// PWM Configuration window closed event Handler. Dispose the unmanaged resources.
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void PWMGenConfigWinform_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/