﻿/**
  ******************************************************************************
  * @file    QdcResetConfigWinform.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file handles the QDC Counter reset configuration dialog box
  ******************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace mPCIe_DIO_24s
{
    public partial class QdcResetConfigWinform : Form
    {        
        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the QdcResetConfigWinform components
        /// </summary>
        public QdcResetConfigWinform()
        {
            InitializeComponent();
        }

        /*************************************************************************************************************
        * 
        * Private Functions
        * 
        *************************************************************************************************************/
        /// <summary>
        /// Quadrature Input Reset configuration Form load event handler. Initializes the form with the default values
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void QdcResetConfigWinform_Load(object sender, EventArgs e)
        {
#if false
// QDC index pin is not supported in Port-A
            // Update the supported reset source pin name
            string[] portAPinNames = new string[] { "PA0" , "PA1" , "PA2" , "PA3" , "PA4", "PA5", "PA6", "PA7" };
            cbRestTrigPin.Items.AddRange(portAPinNames);                     
#endif
            // Get the QDC pin name & number                        
            Tuple<ePortName, ePinNum> pin;
            pin = Main_Form.GetQdcPortPinDetails();
            ePortName qdcPortName = pin.Item1;
            ePinNum qdcPinNumber = pin.Item2;
            
            if (Main_Form.ConfigurationParam[(int)qdcPortName, (int)qdcPinNumber].PinNumber == ePinNum.ePin0)
            {
                // PC0 & PC1 is configured as QDC input. Update the Trigger source combo box
                string[] portCPinNames = new string[] { "PC2" }; // PC0 & PC1 can not be configured as trigger source
                cbRestTrigPin.Items.AddRange(portCPinNames);
            }
            else
            {
                // PC2 & PC3 is configured as QDC input. Update the Trigger source combo box
                string[] portCPinNames = new string[] { "PC1" }; // PC2 & PC3 can not be configured as trigger source
                cbRestTrigPin.Items.AddRange(portCPinNames);
            }

            // Set selected index as PA0
            cbRestTrigPin.SelectedIndex = 0;
        }

        /// <summary>
        /// Ok Button click event handler. Validates and saves the configuration parameters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btOK_Click(object sender, EventArgs e)
        {
            string resetSource;
            eConfigParam resetType = eConfigParam.eNone;

            // Get the reset source pin name
            resetSource = cbRestTrigPin.Text;

            // Get the Reset type from radio button
            if (rbRisingEdge.Checked == true)
                resetType = eConfigParam.eRisingEdge;
            else if (rbFallingEdge.Checked == true)
                resetType = eConfigParam.eFallingEdge;
            else if (rbHighPulse.Checked == true)
                resetType = eConfigParam.eHighPulse;
            else if (rbLowPulse.Checked == true)
                resetType = eConfigParam.eLowPulse;

            // Update the configuration                 
            ucPinConfig.QuadratureControlInputConfigUpdate(resetSource, resetType);

            // Close the configuration dialog
            this.Close();
        }

        /// <summary>
        /// Reset Pin combo box index changed event handler.  Port C only supports Rising edge, hence disable all other trigger type options
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void cbRestTrigPin_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Check if port C is selected
            bool isPortCSelected = false;
            string[] portCPortName = new string[] { "PC0", "PC1", "PC2", "PC3", "PC4", "PC5", "PC6", "PC7" };

            for (int i = 0; i < portCPortName.Length; i++)
            {
                if (cbRestTrigPin.Text == portCPortName[i])
                {
                    // Selected trigger source as Port C, set the flag isPortCSelected
                    isPortCSelected = true;
                    break;
                }
            }
            if (isPortCSelected)
            {
                // Port C only supports Rising edge, hence disable all other radio buttons
                rbFallingEdge.Enabled = false;
                rbHighPulse.Enabled = false;
                rbLowPulse.Enabled = false;
                rbRisingEdge.Checked = true;
            }
            else
            {
                // Enable all the radio buttons
                rbFallingEdge.Enabled = true;
                rbHighPulse.Enabled = true;
                rbLowPulse.Enabled = true;                
            }
        }

        /// <summary>
        /// QDC Configuration window closed event Handler. Dispose the unmanaged resources.
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void QdcResetConfigWinform_FormClosed(object sender, FormClosedEventArgs e)
        {                    
            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);
        }    
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/