﻿/**
  ******************************************************************************
  * @file    StartUpWinForm.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file handles the application start-up form
  ******************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using AIOWDMNet;  // the namespace exposes the AIOWDM Class interface 

namespace mPCIe_DIO_24s
{
    public partial class StartUpWinForm : Form
    {
        /*************************************************************************************************************
         * 
         * Public Variable declarations
         * 
         *************************************************************************************************************/
        private int numberOfCards = 0;
        private static StartUpWinForm startup;

        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the StartUpWinForm components
        /// </summary>
        public StartUpWinForm()
        {
            InitializeComponent();
        }

        /*************************************************************************************************************
        * 
        * Private Functions
        * 
        *************************************************************************************************************/
        /// <summary>
        /// Start-up configuration winform load event handler. Read mPCIe card details and update the dropdown list
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void StartUpWinForm_Load(object sender, EventArgs e)
        {            
            // Assign this to the startup variable
            startup = this;

            // Get the total number of cards installed
            numberOfCards = AIOWDM.GetNumCards();            
            if (numberOfCards <= 0)
            {
                cbDeviceList.Items.Add("No Devices Found");
#if !DEBUG
                btOk.Enabled = false;
#endif
            }
            else
            {
                for (int index = 0; index < numberOfCards; index++)
                {                    
                    cbDeviceList.Items.Add(DeviceInfo.DEVICE_NAME);                    
                }
            }
            cbDeviceList.SelectedIndex = 0;
        }

        /// <summary>
        /// OK Button Click event handler. Load the main form if valid device found.
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btOk_Click(object sender, EventArgs e)
        {            
#if DEBUG
            Main_Form mainFormCtrl = new Main_Form();

            mainFormCtrl.UpdateStartupConfig(cbDeviceList.SelectedIndex);                
            this.Hide();
            mainFormCtrl.Show();
#else
            UInt16 retValue;
            Main_Form mainFormCtrl = new Main_Form();            
            if (numberOfCards > 0)
            {
                // Update the mapping info (Memory mapped or IO mapped)
                retValue = mainFormCtrl.UpdateStartupConfig(cbDeviceList.SelectedIndex);
                if (retValue != 0)
                {
                    MessageBox.Show("Configuration of driver mapping failed, return value: " + retValue.ToString("X"));
                    this.Close();
                }
                else
                {
                    this.Hide();
                    mainFormCtrl.Show();    // Display the Main form                        
                }
            }
            else
            {
                // No Valid device found close the Form
                this.Close();
            }
#endif
        }

        /// <summary>
        /// Cancel Button Click event handler. Close the start-up configuration Form
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Function Closes the Start-up configuration form
        /// </summary>
        public static void CloseStartupConfig()
        {
            startup.Close();
        }

        /// <summary>
        /// Start-up configuration Form Closed Event handler. Dispose the unmanaged resources
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void StartUpWinForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/