﻿/**
  ********************************************************************************
  * @file    TestUIWinform.cs
  * @author  Happiest Minds Firmware Team
  * @version v1.0
  * @date    13-December-2016
  * @brief   This file handles the application Test UI Dialog
  *          NOTE: This is only for testing purpose, Available only in debug mode
  ********************************************************************************
  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using AIOWDMNet;

namespace mPCIe_DIO_24s
{
    public partial class TestUIWinform : Form
    {
        /*************************************************************************************************************
         * 
         * Public Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Initialize the TestUIWinform components
        /// </summary>
        public TestUIWinform()
        {
            InitializeComponent();
        }

        /*************************************************************************************************************
         * 
         * Private Functions
         * 
         *************************************************************************************************************/
        /// <summary>
        /// Text box allows only Hexadecimal values, block all other characters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbWriteAddress_KeyPress(object sender, KeyPressEventArgs e)
        {
            char ch = e.KeyChar;
            if (!((ch >= 'A' && ch <= 'F') || (ch >= 48 && ch <= 57)))
                e.Handled = true;
        }

        /// <summary>
        /// Text box allows only Hexadecimal values, block all other characters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbWriteData_KeyPress(object sender, KeyPressEventArgs e)
        {
            char ch = e.KeyChar;
            if (!((ch >= 'A' && ch <= 'F') || (ch >= 48 && ch <= 57)))
                e.Handled = true;
        }

        /// <summary>
        /// Text box allows only Hexadecimal values, block all other characters
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void tbReadAddress_KeyPress(object sender, KeyPressEventArgs e)
        {
            char ch = e.KeyChar;
            if (!((ch >= 'A' && ch <= 'F') || (ch >= 48 && ch <= 57)))
                e.Handled = true;
        }

        /// <summary>
        /// Write Button Click event handler. Write data to the specified register
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btWrite_Click(object sender, EventArgs e)
        {
            UInt32 address;
            UInt32 data;

            if (tbWriteAddress.Text == "")
                return;

            if (tbWriteData.Text == "")
                return;

            try
            {
                address = UInt32.Parse(tbWriteAddress.Text, System.Globalization.NumberStyles.HexNumber);
                data = UInt32.Parse(tbWriteData.Text, System.Globalization.NumberStyles.HexNumber);
            }
            catch
            {
                MessageBox.Show("Entered data is Invalid");
                return;
            }            

            if (rbByte.Checked == true)
            {
                data = data & 0xFF;
                AIOWDM.RelOutPortB(Main_Form.cardNumber, address, (byte)data);                
                MessageBox.Show("Address: " + address.ToString("X") + " Data(Byte): " + data.ToString("X"), "Success!");
            }                
            else if (rbWord.Checked == true)
            {
                data = data & 0xFFFF;
                AIOWDM.RelOutPort(Main_Form.cardNumber, address, (UInt16)data);                
                MessageBox.Show("Address: " + address.ToString("X") + " Data(Word): " + data.ToString("X"), "Success!");
            }                
            else if (rbLongWord.Checked == true)
            {
                AIOWDM.RelOutPortL(Main_Form.cardNumber, address, data);
                MessageBox.Show("Address: " + address.ToString("X") + " Data(LongWord): " + data.ToString("X"), "Success!");
            }                
        }

        /// <summary>
        /// Read Button Click event handler. Reads data from the specified register
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void btRead_Click(object sender, EventArgs e)
        {
            UInt32 address;
            UInt32 data = 0;

            if (tbReadAddress.Text == "")
            {
                tbReadData.Text = "";
                return;
            }                

            try
            {
                address = UInt32.Parse(tbReadAddress.Text, System.Globalization.NumberStyles.HexNumber);
            }
            catch
            {
                MessageBox.Show("Entered data is Invalid");
                return;
            }
            
            if (rbByte.Checked == true)
            {
                data = AIOWDM.RelInPortB(Main_Form.cardNumber, address);
                data = data & 0xFF;
                tbReadData.Text = "B: " + data.ToString("X");
            }
            else if (rbWord.Checked == true)
            {
                data = AIOWDM.RelInPort(Main_Form.cardNumber, address);
                tbReadData.Text = "W: " + data.ToString("X");
            }
            else if (rbLongWord.Checked == true)
            {
                data = AIOWDM.RelInPortL(Main_Form.cardNumber, address);
                tbReadData.Text = "LW: " + data.ToString("X");
            }            
        }

        /// <summary>
        /// Test UI window closed event Handler. Dispose the unmanaged resources.
        /// </summary>
        /// <param name="sender">The object which has raised the event</param>
        /// <param name="e">Additional information about the event</param>
        private void Winform_TestUI_FormClosed(object sender, FormClosedEventArgs e)
        {             
            // Dispose of unmanaged resources.
            Dispose(true);
            GC.SuppressFinalize(this);
        }    
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/
