unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    Panel2: TPanel;
    GroupBox2: TGroupBox;
    CardCombo: TComboBox;
    GroupBox1: TGroupBox;
    PinCombo: TComboBox;
    GroupBox3: TGroupBox;
    PulseCountEdit: TEdit;
    PulseDurLowGroup: TGroupBox;
    PulseDurHighGroup: TGroupBox;
    PulseDurLowTrack: TTrackBar;
    PulseDurHighTrack: TTrackBar;
    Panel1: TPanel;
    GenerateButton: TButton;
    MainStatus: TStatusBar;
    FlashTime: TTimer;
    LogButton: TButton;
    cbInvert: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure PulseDurLowTrackChange(Sender: TObject);
    procedure PulseDurHighTrackChange(Sender: TObject);
    procedure FlashTimeTimer(Sender: TObject);
    procedure GenerateButtonClick(Sender: TObject);
    procedure LogButtonClick(Sender: TObject);
  protected
    bFailed: Boolean;
    FlashColor: TColor;

    function StatusPrefix(Status: Word): AnsiString;
    function MemI16(CardNum: Integer; Offset: Word; Msg:String = ''): Word;
    function MemI32(CardNum: Integer; Offset: Word; Msg:String = ''): LongWord;
    function MemI8(CardNum: Integer; Offset: Word; Msg:String = ''): Byte;
    procedure MemO16(CardNum: Integer; Offset, Data: Word; Msg:String = '');
    procedure MemO32(CardNum: Integer; Offset: Word; Data: LongWord; Msg:String = '');
    procedure MemO8(CardNum: Integer; Offset: Word; Data: Byte; Msg:String = '');
    function PortI8(CardNum: Integer; Offset: Word; Msg:String = ''): Byte;
    procedure PortO8(CardNum: Integer; Offset: Word; Data: Byte; Msg:String = '');
    procedure Log(LogLine: AnsiString);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math, AIOWDM, LogUnit;

const
  FlashMax = 256;
  logdelim = #9;
  DO_asPulseTrain = $00010000;  

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;

function FadeColor(A, B: TColor; BNess: Single): TColor;
type
  TRGBFColor = packed record
    case Boolean of
    True: (
      Color: TColor;
    );
    False: (
      Red: Byte;
      Green: Byte;
      Blue: Byte;
      Flags: Byte;
    );
  end;

var
  AC: TRGBFColor;
  BC: TRGBFColor;
  RC: TRGBFColor absolute Result;
  ANess: Single;

begin
  AC.Color := ColorToRGB(A);
  BC.Color := ColorToRGB(B);
  ANess := 1 - BNess;
  RC.Red   := Round(ANess * AC.Red   + BNess * BC.Red  );
  RC.Green := Round(ANess * AC.Green + BNess * BC.Green);
  RC.Blue  := Round(ANess * AC.Blue  + BNess * BC.Blue );
  RC.Flags := 0;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  iCardNum: LongInt;
  DeviceID, NomSize: LongWord;
  Nom: AnsiString;

begin
  LoadDefaultFont(Font);

  Caption := FloatToStr(High(Int64));

  for iCardNum := 0 to GetNumCards-1 do begin
    NomSize := 256;
    SetLength(Nom, NomSize);
    if QueryCardInfo(iCardNum, @DeviceID, nil, @NomSize, @Nom[1]) = 0 then begin
      CardCombo.Items.Add('[' + IntToStr(iCardNum) + '] ???');
    end
    else begin
      if NomSize > 0 then Dec(NomSize);
      SetLength(Nom, NomSize);
      CardCombo.Items.Add('[' + IntToStr(iCardNum) + '] ' + Nom + ' [' + IntToHex(DeviceID, 4) + ']');
    end;
  end;

  if CardCombo.Items.Count <> 0 then CardCombo.ItemIndex := 0;
  PinCombo.ItemIndex := 0;
  PulseDurLowTrackChange(PulseDurLowTrack);
  PulseDurHighTrackChange(PulseDurHighTrack);
end;

function ExpressIntervalCounts(Ct: Word): AnsiString;
begin
  Result := IntToStr(Ct) + ' (' + FloatToStr(8/1000 * Ct) + 's)';
  //if Pos('E', Result) = 0 then Exit;
end;

procedure TMainForm.PulseDurLowTrackChange(Sender: TObject);
begin
  PulseDurLowGroup.Caption := 'Pulse Duration Low: ' + ExpressIntervalCounts(PulseDurLowTrack.Position);
end;

procedure TMainForm.PulseDurHighTrackChange(Sender: TObject);
begin
  PulseDurHighGroup.Caption := 'Pulse Duration High: ' + ExpressIntervalCounts(PulseDurHighTrack.Position);
end;

procedure TMainForm.FlashTimeTimer(Sender: TObject);
var
  I: Integer;
  F: Single;

begin
  I := FlashTime.Tag;
  if I <= 0 then begin
    FlashTime.Enabled := False;
    MainStatus.ParentColor := True;
    MainStatus.ParentFont := True;
    Exit;
  end;

  Dec(I, 7);
  if I < 0 then I := 0;
  FlashTime.Tag := I;
  F := Sqr(I/FlashMax);
  MainStatus.Color := FadeColor(Color, FlashColor, F);
end;

procedure TMainForm.GenerateButtonClick(Sender: TObject);
var
  CardNum, Ct, iBit, LowDur, HighDur: Integer;
  BitOff: Word;
  ReportStr: AnsiString;
  fActiveHigh: Longword;

begin
  MainStatus.Panels[0].Text := '';

  CardNum := CardCombo.ItemIndex;
  if CardNum < 0 then Exit;

  Ct := StrToInt(PulseCountEdit.Text) and $FFFF;
  if Ct <= 0 then Exit;

  iBit := PinCombo.ItemIndex;
  if iBit < 0 then Exit;

  LowDur := PulseDurLowTrack.Position and $FFFF;
  HighDur := PulseDurHighTrack.Position and $FFFF;
  fActiveHigh := $00080000;
  if cbInvert.Checked then fActiveHigh := 0;

  BitOff := (1 + 8 + iBit) * $100;

  bFailed := False;
  Log('Card[' + IntToStr(CardNum) + ']');
  PortO8(CardNum, $3, $80);
  MemO32(CardNum, BitOff + $00, DO_asPulseTrain OR fActiveHigh);
  MemO32(CardNum, BitOff + $1C, LowDur or (HighDur shl 16));
  MemO32(CardNum, BitOff + $20, Ct);
  MemO32(CardNum, $3C, ($10000 shl iBit) or MemI32(CardNum, $3C));

  ReportStr := IntToStr(Ct) + ' pulses, ' + IntToStr(LowDur) + '/' + IntToStr(HighDur) + ', on pin ' + PinCombo.Items[iBit];
  if bFailed then begin
    ReportStr := 'Failed to generate ' + ReportStr;
    FlashColor := $0000FF;
  end
  else begin
    ReportStr := 'Generated ' + ReportStr;
    FlashColor := $00FF00;
  end;
  MainStatus.Panels[0].Text := ReportStr;

  MainStatus.Color := FlashColor;
  FlashTime.Tag := FlashMax;
  FlashTime.Enabled := True;
end;

function TMainForm.StatusPrefix(Status: Word): AnsiString;
begin
  if Status = 0 then
    Result := ''
  else if Status = $AA55 then begin
    Result := 'FAIL: ';
    bFailed := True;
  end
  else begin
    Result := 'Error ' + IntToHex(Status, 1) + ' on ';
    bFailed := True;
  end
end;

function TMainForm.PortI8(CardNum: LongInt; Offset: Word; Msg:String): Byte;
begin
  RelChangeMapping(CardNum, True);
  Result := RelInPortB(CardNum, Offset);
  Log('I/O +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 2) + logdelim + Msg);
end;

procedure TMainForm.PortO8(CardNum: LongInt; Offset: Word; Data: Byte; Msg:String);
begin
  RelChangeMapping(CardNum, True);
  Log(StatusPrefix(RelOutPortB(CardNum, Offset, Data)) + 'I/O +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 2) + logdelim + Msg);
end;

//PortO16 and PortO32 don't work on this card, for some reason.

function TMainForm.MemI8(CardNum: LongInt; Offset: Word; Msg:String): Byte;
begin
  RelChangeMapping(CardNum, False);
  Result := RelInPortB(CardNum, Offset);
  Log('Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 2) + logdelim + Msg);
end;

function TMainForm.MemI16(CardNum: LongInt; Offset: Word; Msg:String): Word;
begin
  RelChangeMapping(CardNum, False);
  Result := RelInPort(CardNum, Offset);
  Log('Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 4) + logdelim + Msg);
end;

function TMainForm.MemI32(CardNum: LongInt; Offset: Word; Msg:String): LongWord;
begin
  RelChangeMapping(CardNum, False);
  Result := RelInPortL(CardNum, Offset);
  Log('Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 8) + logdelim + Msg);
end;

procedure TMainForm.MemO8(CardNum: LongInt; Offset: Word; Data: Byte; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  Log(StatusPrefix(RelOutPortB(CardNum, Offset, Data)) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 2) + logdelim + Msg);
end;

procedure TMainForm.MemO16(CardNum: LongInt; Offset: Word; Data: Word; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  Log(StatusPrefix(RelOutPort(CardNum, Offset, Data)) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 4) + logdelim + Msg);
end;

procedure TMainForm.MemO32(CardNum: LongInt; Offset: Word; Data: LongWord; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  Log(StatusPrefix(RelOutPortL(CardNum, Offset, Data)) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 8) + logdelim + Msg);
end;

procedure TMainForm.LogButtonClick(Sender: TObject);
begin
  LogForm.Visible := True;
end;

procedure TMainForm.Log(LogLine: AnsiString);
begin
  LogForm.LogMemo.Lines.Add(LogLine);
end;

end.
