unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    Panel2: TPanel;
    GroupBox2: TGroupBox;
    CardCombo: TComboBox;
    GroupBox1: TGroupBox;
    PinCombo: TComboBox;
    PulseDurLowGroup: TGroupBox;
    PulseDurHighGroup: TGroupBox;
    PulseDurLowTrack: TTrackBar;
    PulseDurHighTrack: TTrackBar;
    Panel1: TPanel;
    GenerateButton: TButton;
    MainStatus: TStatusBar;
    FlashTime: TTimer;
    cbInvert: TCheckBox;
    cbAutoOutMode: TCheckBox;
    cbPWM: TCheckBox;
    PulseCountGroup: TGroupBox;
    PulseCountTrack: TTrackBar;
    LogGroup: TGroupBox;
    LogMemo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure PulseDurLowTrackChange(Sender: TObject);
    procedure PulseDurHighTrackChange(Sender: TObject);
    procedure FlashTimeTimer(Sender: TObject);
    procedure GenerateButtonClick(Sender: TObject);
    procedure PinComboChange(Sender: TObject);
    procedure cbAutoOutModeClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PulseCountTrackChange(Sender: TObject);
    procedure cbPWMClick(Sender: TObject);
    procedure CardComboChange(Sender: TObject);
  private
    BitOff: Word;
    LowDur, HighDur, Ct, CardNum: Integer;
    procedure UpdatePulseConfig;
  protected
    bFailed: Boolean;
    FlashColor: TColor;

    function StatusPrefix(Status: Word): AnsiString;
    function MemI16(CardNum: Integer; Offset: Word; Msg:String = ''): Word;
    function MemI32(CardNum: Integer; Offset: Word; Msg:String = ''): LongWord;
    function MemI8(CardNum: Integer; Offset: Word; Msg:String = ''): Byte;
    procedure MemO16(CardNum: Integer; Offset, Data: Word; Msg:String = '');
    procedure MemO32(CardNum: Integer; Offset: Word; Data: LongWord; Msg:String = '');
    procedure MemO8(CardNum: Integer; Offset: Word; Data: Byte; Msg:String = '');
    procedure Log(LogLine: AnsiString);
    procedure UpdatePulseCount;
    procedure UpdatePulseDurations;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math, AIOWDM;

const
  FlashMax = 256;
  logdelim = #9;
  bmDO_asPulseTrain = $00010000;
  bmDO_asPWM = $00020000;
  bmAutoOutMode =  $00000001;

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;

function FadeColor(A, B: TColor; BNess: Single): TColor;
type
  TRGBFColor = packed record
    case Boolean of
    True: (
      Color: TColor;
    );
    False: (
      Red: Byte;
      Green: Byte;
      Blue: Byte;
      Flags: Byte;
    );
  end;

var
  AC: TRGBFColor;
  BC: TRGBFColor;
  RC: TRGBFColor absolute Result;
  ANess: Single;

begin
  AC.Color := ColorToRGB(A);
  BC.Color := ColorToRGB(B);
  ANess := 1 - BNess;
  RC.Red   := Round(ANess * AC.Red   + BNess * BC.Red  );
  RC.Green := Round(ANess * AC.Green + BNess * BC.Green);
  RC.Blue  := Round(ANess * AC.Blue  + BNess * BC.Blue );
  RC.Flags := 0;
end;

function ExpressIntervalCounts(Ct: Word): AnsiString;
begin
  Result := IntToStr(Ct) + ' (' + FloatToStr(8/1000 * Ct) + 's)';
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  iCardNum: LongInt;
  DeviceID, NomSize: LongWord;
  Nom: AnsiString;
begin
  LoadDefaultFont(Font);

  for iCardNum := 0 to GetNumCards-1 do begin
    NomSize := 256;
    SetLength(Nom, NomSize);
    if QueryCardInfo(iCardNum, @DeviceID, nil, @NomSize, @Nom[1]) = 0 then begin
      CardCombo.Items.Add('[' + IntToStr(iCardNum) + '] ???');
    end
    else begin
      if NomSize > 0 then Dec(NomSize);
      SetLength(Nom, NomSize);
      CardCombo.Items.Add('[' + IntToStr(iCardNum) + '] ' + Nom + ' [' + IntToHex(DeviceID, 4) + ']');
    end;
  end;

  if CardCombo.Items.Count <> 0 then CardCombo.ItemIndex := 0;
  PinCombo.ItemIndex := 0;
end;

procedure TMainForm.FormShow(Sender: TObject);
  var
    valueRead: Longword;
    bAutoOut: Boolean;
begin
  CardNum := -1; // inhibit the change handlers from writing to the device

  // cause event handlers to calculate and display human-readable initial values
  PulseDurLowTrackChange(PulseDurLowTrack);
  PulseDurHighTrackChange(PulseDurHighTrack);
  PulseCountTrackChange(PulseCountTrack);
  cbAutoOutModeClick(cbAutoOutMode);

  if CardCombo.ItemIndex < 0 then exit; // don't write anything if no card found!

  valueRead := MemI32(CardCombo.ItemIndex, $2D, 'read "auto mode" configuration bit (D0)');
  bAutoOut := (valueRead and bmAutoOutMode) > 0; // is card in Auto Output Mode?
  cbAutoOutMode.Checked := bAutoOut;  // CardNum still -1 so change handler won't write to the device

  CardNum := CardCombo.ItemIndex; // now update CardNum so future changes get written to the device
  CardComboChange(CardCombo); // configure card
  PinComboChange(PinCombo);  // configure pin

end;

procedure TMainForm.UpdatePulseCount;
begin
  if Ct = PulseCountTrack.Position then exit;

  Ct := PulseCountTrack.Position;

  MemO32(CardNum, BitOff + $08, Ct, 'Update Pulse Count'); // in "auto out mode" this step alone triggers pulse outputs to start
end;

procedure TMainForm.UpdatePulseConfig;
  var
    fActiveHigh: Longword;
    fPWM: Longword;
begin
  fActiveHigh := $00080000;
  if cbInvert.Checked then fActiveHigh := 0;

  fPWM := bmDO_asPulseTrain;
  if cbPWM.Checked then fPWM := bmDO_asPWM;

  MemO32(CardNum, BitOff + $10, fPWM OR fActiveHigh, 'configure bit as OPTG/PWM etc');

  if cbAutoOutMode.Checked then
    MemO32(CardNum, $2D, bmAutoOutMode, 'Turn AOP on')
  else
    MemO32(CardNum, $2D, 0, 'Turn AOP off')
end;

procedure TMainForm.UpdatePulseDurations;
begin
  LowDur := PulseDurLowTrack.Position;
  HighDur := PulseDurHighTrack.Position;

  MemO32(CardNum, BitOff + $20, LowDur, 'Write Low pulse duration');
  MemO32(CardNum, BitOff + $24, HighDur, 'Write High pulse duration');
end;

procedure TMainForm.PulseDurLowTrackChange(Sender: TObject);
begin
  if LowDur = PulseDurLowTrack.Position then exit;

  PulseDurLowGroup.Caption := 'Pulse Duration Low: ' + ExpressIntervalCounts(PulseDurLowTrack.Position);
  UpdatePulseDurations;
end;

procedure TMainForm.PulseDurHighTrackChange(Sender: TObject);
begin
  if HighDur = PulseDurHighTrack.Position then exit;

  PulseDurHighGroup.Caption := 'Pulse Duration High: ' + ExpressIntervalCounts(PulseDurHighTrack.Position);
  UpdatePulseDurations;
end;

procedure TMainForm.PulseCountTrackChange(Sender: TObject);
begin
  if Ct = PulseCountTrack.Position then exit;

  PulseCountGroup.Caption := 'Pulse Count: ' + IntToStr(PulseCountTrack.Position);
  UpdatePulseCount;
end;

procedure TMainForm.FlashTimeTimer(Sender: TObject);
var
  I: Integer;
  F: Single;

begin
  I := FlashTime.Tag;
  if I <= 0 then begin
    FlashTime.Enabled := False;
    MainStatus.ParentColor := True;
    MainStatus.ParentFont := True;
    Exit;
  end;

  Dec(I, 7);
  if I < 0 then I := 0;
  FlashTime.Tag := I;
  F := Sqr(I/FlashMax);
  MainStatus.Color := FadeColor(Color, FlashColor, F);
end;

procedure TMainForm.GenerateButtonClick(Sender: TObject);
  var
    Ct, iBit: Integer;
    ReportStr: AnsiString;
begin
  MainStatus.Panels[0].Text := '';

  Log('Generate Pulse(s)');

  Ct := PulseCountTrack.Position;
  if Ct <= 0 then Exit;

  iBit := PinCombo.ItemIndex;
  if iBit < 0 then Exit;

  bFailed := False;
  UpdatePulseCount;
  UpdatePulseDurations;
  MemO32(CardNum, $50, ($10000 shl iBit) or MemI32(CardNum, $50), 'Go Bit - starts pulse train');

  ReportStr := IntToStr(Ct) + ' pulses, ' + IntToStr(LowDur) + '/' + IntToStr(HighDur) + ', on pin ' + PinCombo.Items[iBit];
  if bFailed then begin
    ReportStr := 'Failed to generate ' + ReportStr;
    FlashColor := $0000FF;
  end
  else begin
    ReportStr := 'Generated ' + ReportStr;
    FlashColor := $00FF00;
  end;
  MainStatus.Panels[0].Text := ReportStr;

  MainStatus.Color := FlashColor;
  FlashTime.Tag := FlashMax;
  FlashTime.Enabled := True;
end;

procedure TMainForm.PinComboChange(Sender: TObject);
begin
  cbAutoOutMode.Enabled := PinCombo.ItemIndex < 8; // AOP, "Auto Out Pulse" mode is only supported on Port B
  BitOff := (1 + 8 + PinCombo.ItemIndex) * $100;  // +8 because PinCombo only lists B0-C7, no Port A bits
  UpdatePulseConfig;
end;

procedure TMainForm.cbAutoOutModeClick(Sender: TObject);
begin
  GenerateButton.Enabled := not cbAutoOutMode.Checked;

  UpdatePulseConfig;
end;

procedure TMainForm.cbPWMClick(Sender: TObject);
begin
  PulseCountTrack.Visible := not cbPWM.checked;
  if cbPWM.checked then
    PulseCountGroup.Caption := 'Pulse Count: Forever '
  else
    PulseCountGroup.Caption := 'Pulse Count: ' + IntToStr(PulseCountTrack.Position) + ' ';
end;

procedure TMainForm.CardComboChange(Sender: TObject);
begin
  CardNum := CardCombo.ItemIndex;

  MemO8(CardNum, $3, MemI8(CardNum, $3) and not $B, 'ports B&C as output');
end;

function TMainForm.StatusPrefix(Status: Word): AnsiString;
begin
  if Status = 0 then
    Result := ''
  else if Status = $AA55 then begin
    Result := 'FAIL: ';
    bFailed := True;
  end
  else begin
    Result := 'Error ' + IntToHex(Status, 1) + ' on ';
    bFailed := True;
  end
end;

function Card(C: Integer): String;
begin
  Result := '[CardNum ' + IntToStr(C) + '] ';
end;

function TMainForm.MemI8(CardNum: LongInt; Offset: Word; Msg:String): Byte;
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Result := RelInPortB(CardNum, Offset);
  Log(Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 2) + logdelim + Msg);
end;

function TMainForm.MemI16(CardNum: LongInt; Offset: Word; Msg:String): Word;
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Result := RelInPort(CardNum, Offset);
  Log(Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 4) + logdelim + Msg);
end;

function TMainForm.MemI32(CardNum: LongInt; Offset: Word; Msg:String): LongWord;
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Result := RelInPortL(CardNum, Offset);
  Log(Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 8) + logdelim + Msg);
end;

procedure TMainForm.MemO8(CardNum: LongInt; Offset: Word; Data: Byte; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Log(StatusPrefix(RelOutPortB(CardNum, Offset, Data)) + Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 2) + logdelim + Msg)
  else
    Log('demo mode ' + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 2) + logdelim + Msg);
end;

procedure TMainForm.MemO16(CardNum: LongInt; Offset: Word; Data: Word; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Log(StatusPrefix(RelOutPort(CardNum, Offset, Data)) + Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 4) + logdelim + Msg)
  else
    Log('demo mode ' + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 4) + logdelim + Msg);

end;

procedure TMainForm.MemO32(CardNum: LongInt; Offset: Word; Data: LongWord; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Log(StatusPrefix(RelOutPortL(CardNum, Offset, Data)) + Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 8) + logdelim + Msg)
  else
    Log('demo mode ' + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 8) + logdelim + Msg)
end;

procedure TMainForm.Log(LogLine: AnsiString);
begin
  LogMemo.Lines.Add(LogLine);
end;

end.
