unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    Panel2: TPanel;
    GroupBox2: TGroupBox;
    CardCombo: TComboBox;
    MotorControlFrequencyGroup: TGroupBox;
    MotorControlCountGroup: TGroupBox;
    MotorControlFrequencyTrack: TTrackBar;
    MotorControlCountTrack: TTrackBar;
    Panel1: TPanel;
    GenerateButton: TButton;
    MainStatus: TStatusBar;
    FlashTime: TTimer;
    LogButton: TButton;
    cbMCS1_DIR: TCheckBox;
    cbMCS1_CTRL: TCheckBox;
    cbMCS1_SEL: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure MotorControlFrequencyTrackChange(Sender: TObject);
    procedure MotorControlCountTrackChange(Sender: TObject);
    procedure FlashTimeTimer(Sender: TObject);
    procedure GenerateButtonClick(Sender: TObject);
    procedure LogButtonClick(Sender: TObject);
  protected
    bFailed: Boolean;
    FlashColor: TColor;

    function StatusPrefix(Status: Word): AnsiString;
    function MemI16(CardNum: Integer; Offset: Word; Msg:String = ''): Word;
    function MemI32(CardNum: Integer; Offset: Word; Msg:String = ''): LongWord;
    function MemI8(CardNum: Integer; Offset: Word; Msg:String = ''): Byte;
    procedure MemO16(CardNum: Integer; Offset, Data: Word; Msg:String = '');
    procedure MemO32(CardNum: Integer; Offset: Word; Data: LongWord; Msg:String = '');
    procedure MemO8(CardNum: Integer; Offset: Word; Data: Byte; Msg:String = '');
    procedure Log(LogLine: AnsiString);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math, AIOWDM, LogUnit;

const
  FlashMax = 256;
  logdelim = #9;
  secPerCount = 16e-9; //Double the expected. Maybe because the same value determines both high side and low side?
  DO_asPulseTrain = $00010000;  

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;

function FadeColor(A, B: TColor; BNess: Single): TColor;
type
  TRGBFColor = packed record
    case Boolean of
    True: (
      Color: TColor;
    );
    False: (
      Red: Byte;
      Green: Byte;
      Blue: Byte;
      Flags: Byte;
    );
  end;

var
  AC: TRGBFColor;
  BC: TRGBFColor;
  RC: TRGBFColor absolute Result;
  ANess: Single;

begin
  AC.Color := ColorToRGB(A);
  BC.Color := ColorToRGB(B);
  ANess := 1 - BNess;
  RC.Red   := Round(ANess * AC.Red   + BNess * BC.Red  );
  RC.Green := Round(ANess * AC.Green + BNess * BC.Green);
  RC.Blue  := Round(ANess * AC.Blue  + BNess * BC.Blue );
  RC.Flags := 0;
end;

function Sgn(I: Integer): Integer; overload;
begin
  if I = 0 then
    Result := 0
  else if I < 0 then
    Result := -1
  else
    Result := 1
  ;
end;

function ModFromInt(I: Integer): AnsiString;
begin
  if I = 0 then
    Result := ''
  else if I < 0 then
    Result := StringOfChar('-', -I)
  else //if I > 0 then
    Result := StringOfChar('+', I)
  ;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  iCardNum: LongInt;
  DeviceID, NomSize: LongWord;
  Nom: AnsiString;

begin
  LoadDefaultFont(Font);

  for iCardNum := 0 to GetNumCards-1 do begin
    NomSize := 256;
    SetLength(Nom, NomSize);
    if QueryCardInfo(iCardNum, @DeviceID, nil, @NomSize, @Nom[1]) = 0 then begin
      CardCombo.Items.Add('[' + IntToStr(iCardNum) + '] ???');
    end
    else begin
      if NomSize > 0 then Dec(NomSize);
      SetLength(Nom, NomSize);
      CardCombo.Items.Add('[' + IntToStr(iCardNum) + '] ' + Nom + ' [' + IntToHex(DeviceID, 4) + ']');
    end;
  end;

  if CardCombo.Items.Count <> 0 then CardCombo.ItemIndex := 0;

  MotorControlFrequencyTrackChange(MotorControlFrequencyTrack);
  MotorControlCountTrackChange(MotorControlCountTrack);
end;

function ExpressIntervalCounts(Ct: Word): AnsiString;
begin
  Result := IntToStr(Ct) + ' (' + FloatToStrF(8/1000 * Ct, ffFixed, 5, 6) + 's)';
end;

function ExpressFrequencyDivisor(Divisor: LongWord): AnsiString;
begin
  Result := FloatToStrF(1 / (Divisor*secPerCount*1000), ffFixed, 5, 6) + 'kHz';
end;

procedure TMainForm.MotorControlFrequencyTrackChange(Sender: TObject);
begin
  MotorControlFrequencyGroup.Caption := 'Freq: ' + ExpressFrequencyDivisor(MotorControlFrequencyTrack.Position) + ', divisor ' + IntToStr(MotorControlFrequencyTrack.Position);
end;

procedure TMainForm.MotorControlCountTrackChange(Sender: TObject);
begin
  MotorControlCountGroup.Caption := 'Count: ' + IntToStr(MotorControlCountTrack.Position);
end;

procedure TMainForm.FlashTimeTimer(Sender: TObject);
var
  I: Integer;
  F: Single;

begin
  I := FlashTime.Tag;
  if I <= 0 then begin
    FlashTime.Enabled := False;
    MainStatus.ParentColor := True;
    MainStatus.ParentFont := True;
    Exit;
  end;

  Dec(I, 7);
  if I < 0 then I := 0;
  FlashTime.Tag := I;
  F := Sqr(I/FlashMax);
  MainStatus.Color := FadeColor(Color, FlashColor, F);
end;

procedure TMainForm.GenerateButtonClick(Sender: TObject);
var
  CardNum, Ct: Integer;
  ReportStr: AnsiString;
  MCS1Config, MCS1_CTRL, MCS1_SEL, MCS1_DIR, MCS1_DON, FreqDiv: Longword;

begin
  MainStatus.Panels[0].Text := '';

  CardNum := CardCombo.ItemIndex;

  MCS1_SEL := ord(cbMCS1_SEL.checked) * $00000010;
  MCS1_CTRL := ord(cbMCS1_CTRL.checked) * $00000020;
  MCS1_DIR := ord(cbMCS1_DIR.checked) * $00000040;
  MCS1_DON := $00000080;

  FreqDiv := MotorControlFrequencyTrack.Position;
  Ct := MotorControlCountTrack.Position;

  bFailed := False;
  Log('Card[' + IntToStr(CardNum) + ']');
  MemO8(CardNum, $3, $80, 'All ports output');
  MemO32(CardNum, $2100, 0, 'Turn off all advanced features on C4');
  MemO32(CardNum, $2200, 0, 'Turn off all advanced features on C5');
  MemO32(CardNum, $74, FreqDiv, 'MCS Frequency ' + ExpressFrequencyDivisor(FreqDiv));
  MemO32(CardNum, $78, Abs(Ct), 'MCS Count ' + IntToStr(Ct));

  MCS1Config := MCS1_SEL or MCS1_CTRL or MCS1_DIR or MCS1_DON;
  MemO32(CardNum, $70, MCS1Config, 'Enable MotorControl outputs on bits C4&5' );


  ReportStr := IntToStr(Ct) + ' pulses, at ' + ExpressFrequencyDivisor(FreqDiv) + ', on pins C4&C5 ';
  if bFailed then begin
    ReportStr := 'Failed to generate ' + ReportStr;
    FlashColor := $0000FF;
  end
  else begin
    ReportStr := 'Generated ' + ReportStr;
    FlashColor := $00FF00;
  end;
  MainStatus.Panels[0].Text := ReportStr;

  MainStatus.Color := FlashColor;
  FlashTime.Tag := FlashMax;
  FlashTime.Enabled := True;
end;

function TMainForm.StatusPrefix(Status: Word): AnsiString;
begin
  if Status = 0 then
    Result := ''
  else if Status = $AA55 then begin
    Result := 'FAIL: ';
    bFailed := True;
  end
  else begin
    Result := 'Error ' + IntToHex(Status, 1) + ' on ';
    bFailed := True;
  end
end;

function Card(C: Integer): String;
begin
  Result := '[CardNum ' + IntToStr(C) + '] ';
end;

function TMainForm.MemI8(CardNum: LongInt; Offset: Word; Msg:String): Byte;
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Result := RelInPortB(CardNum, Offset);
  Log(Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 2) + logdelim + Msg);
end;

function TMainForm.MemI16(CardNum: LongInt; Offset: Word; Msg:String): Word;
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Result := RelInPort(CardNum, Offset);
  Log(Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 4) + logdelim + Msg);
end;

function TMainForm.MemI32(CardNum: LongInt; Offset: Word; Msg:String): LongWord;
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Result := RelInPortL(CardNum, Offset);
  Log(Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' > ' + IntToHex(Result, 8) + logdelim + Msg);
end;

procedure TMainForm.MemO8(CardNum: LongInt; Offset: Word; Data: Byte; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Log(StatusPrefix(RelOutPortB(CardNum, Offset, Data)) + Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 2) + logdelim + Msg)
  else
    Log('demo mode ' + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 2) + logdelim + Msg);
end;

procedure TMainForm.MemO16(CardNum: LongInt; Offset: Word; Data: Word; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Log(StatusPrefix(RelOutPort(CardNum, Offset, Data)) + Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 4) + logdelim + Msg)
  else
    Log('demo mode ' + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 4) + logdelim + Msg);

end;

procedure TMainForm.MemO32(CardNum: LongInt; Offset: Word; Data: LongWord; Msg:String);
begin
  RelChangeMapping(CardNum, False);
  if CardNum <> -1 then
    Log(StatusPrefix(RelOutPortL(CardNum, Offset, Data)) + Card(CardNum) + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 8) + logdelim + Msg)
  else
    Log('demo mode ' + 'Mem +' + IntToHex(Offset, 1) + ' < ' + IntToHex(Data, 8) + logdelim + Msg)
end;

procedure TMainForm.Log(LogLine: AnsiString);
begin
  LogForm.LogMemo.Lines.Add(LogLine);
end;

procedure TMainForm.LogButtonClick(Sender: TObject);
begin
  LogForm.show;
end;

end.
